import org.w3c.dom.*;
import com.ibm.xml.parsers.*;
import com.ibm.xml.framework.*;

public class Conversion
{
   public static void main(String[] args)
      throws Exception
   {
      if(args.length < 2)
      {
         System.out.println("java Conversion filename rate");
         return;
      }
      double rate = Double.valueOf(args[1]).doubleValue();

      Document document = parse(args[0]);
      Element topLevel = document.getDocumentElement();
      walkNode(topLevel,rate);
   }

   protected static Document parse(String uri)
      throws Exception
   {
      NonValidatingDOMParser parser =
         new NonValidatingDOMParser();
      parser.parse(uri);
      return parser.getDocument();
   }

   protected static void walkNode(Node node,double rate)
   {
      if(node.getNodeType() == Node.ELEMENT_NODE)
      {
         if(node.getNodeName().equals("product"))
            walkProduct((Element)node,rate);
         else
         {
             NodeList children = node.getChildNodes();
             for(int i = 0;i < children.getLength();i++)
                walkNode(children.item(i),rate);
         }
      }
   }

   protected static void walkProduct(Element element,double rate)
   {
      if(element.getNodeName().equals("product"))
      {
         String name = null;
         double price = 0.0;
		 NodeList children = element.getChildNodes();
         for(int i = 0;i < children.getLength();i++)
         {
            Node child = children.item(i);
            if(child.getNodeType() == Node.ELEMENT_NODE)
            {
               if(child.getNodeName().equals("price"))
               {
                  String st = getText(child);
                  price = Double.valueOf(st).doubleValue();
                  price *= rate;
               }
               else if(child.getNodeName().equals("name"))
                  name = getText(child);
            }
         }
	     System.out.println(name + ": " + price);
      }
   }

   protected static String getText(Node node)
   {
      Node child = node.getFirstChild();
      if(child != null && child.getNodeType() == Node.TEXT_NODE)
      {
         Text text = (Text)child;
         return text.getData();
      }
      else
         return null;
   }
}