/*
 * XML By Example, chapter 8: SAX
 */

package com.psol.xbe;

import org.xml.sax.*;
import org.xml.sax.helpers.ParserFactory;

/**
 * SAX event handler to find the cheapest offering
 * in a list of prices.
 * @author bmarchal@pineapplesoft.com
 */

public class Cheapest
   extends HandlerBase
{
   /*
    * event handler
    */

   /**
    * properties we are collecting: cheapest price
    */
   protected double min = Double.MAX_VALUE;

   /**
    * properties we are collecting: cheapest vendor
    */
   protected String vendor = null;

   /**
    * startElement event: the price list is stored as price
    * elements with price and vendor attributes
    * @param name element's name
    * @param attributes element's attributes
    */
   public void startElement(String name,AttributeList attributes)
   {
      if(name.equals("price"))
      {
         String attribute = attributes.getValue("price");
         if(null != attribute)
         {
            double price = toDouble(attribute);
            if(min > price)
            {
               min = price;
               vendor = attributes.getValue("vendor");
            }
         }
      }
   }

   /**
    * helper method: turn a string in a double
    * @param string number as a string
    * @return the number as a double, or 0.0 if it cannot convert
    * the number
    */
   protected double toDouble(String string)
   {
      Double stringDouble = Double.valueOf(string);
      if(null != stringDouble)
         return stringDouble.doubleValue();
      else
         return 0.0;
   }

   /**
    * property accessor: vendor name
    * @return the vendor with the cheapest offer so far
    */
   public String getVendor()
   {
      return vendor;
   }

   /**
    * property accessor: best price
    * @return the best price so far
    */
   public double getMinimum()
   {
      return min;
   }

   /*
    * main() method and properties
    */

   /**
    * the parser class (IBM's XML for Java)
    */
   protected static final String 
      PARSER_NAME = "com.ibm.xml.parsers.SAXParser";

   /**
    * main() method
    * decodes command-line parameters and invokes the parser
Ben: Edits to comment line above okay? Kezia
Yes --ben
    * @param args command-line argument
    * @throw Exception catch-all for underlying exceptions
    */
   public static void main(String[] args)
      throws Exception
   {
      // command-line arguments
      if(args.length < 1)
      {
         System.out.println("java com.psol.xbe.CheapestCL filename");
         return;
      }

      // creates the event handler
      Cheapest cheapest = new Cheapest();

      // creates the parser
      Parser parser = ParserFactory.makeParser(PARSER_NAME);
      parser.setDocumentHandler(cheapest);

      // invokes the parser against the price list
      parser.parse(args[0]);

      // prints the results
      System.out.println("The cheapest offer is " + 
                         cheapest.getVendor() +
                         " ($" + cheapest.getMinimum() + ')');
   }
}