package com.psol.xcommerce;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Shop manages the shop, using XSL for any output.
 *
 * @version Sep 10, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class Shop
   extends HttpServlet
{
   /**
    * the merchant list and the shopping cart
    */
   protected MerchantCollection merchants;
   protected Comlet checkout;

   /**
    * return the list of merchants
    */
   public MerchantCollection getMerchants()
   {
      return merchants;
   }

   /**
    * initializes the servlet
    */
   public void init()
      throws ServletException
   {
      merchants = new MerchantCollection(this);
      checkout = new Checkout(this);
   }

   /**
    * handle GET request
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   protected void doGet(HttpServletRequest request,
                        HttpServletResponse response)
      throws IOException, ServletException
   {
      Comlet comlet = translateURL(request);
      if(null != comlet)
         comlet.doGet(request,response);
      else
         response.sendError(HttpServletResponse.SC_NOT_FOUND);
   }

   /**
    * handle POST request
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   protected void doPost(HttpServletRequest request,
                         HttpServletResponse response)
      throws IOException, ServletException
   {
      Comlet comlet = translateURL(request);
      if(null != comlet)
         comlet.doPost(request,response);
      else
         response.sendError(HttpServletResponse.SC_NOT_FOUND);
   }

   /**
    * Returns the time it was last modified.
    * @param request the request received from the client
    * @return time in milliseconds since January 1, 1970 midnight
    */
   protected long getLastModified(HttpServletRequest request)
   {
      try
      {
         Comlet comlet = translateURL(request);
         if(null != comlet)
            return comlet.getLastModified();
         else
            return -1;
      }
      catch(ServletException e)
      {
         return -1;
      }
   }

   /**
    * decode the URL and select the Comlet that will
    * handle the request
    * @param request the request received from the client
    * @return the Comlet identified in the request
    * @exception problem loading one of the object
    */
   protected Comlet translateURL(HttpServletRequest request)
      throws ServletException
   {
      String merchantID = null,
             pathInfo = request.getPathInfo();
      StringTokenizer tokenizer = null;
      if(null != pathInfo)
      {
         tokenizer = new StringTokenizer(pathInfo,"/");
         if(tokenizer.hasMoreTokens())
            merchantID = tokenizer.nextToken();
      }
      if(null == merchantID)
         return merchants;
      if(merchantID.equals("checkout"))
         return checkout;
      Merchant merchant = merchants.getMerchant(merchantID);
      String productID = null;
      if(tokenizer.hasMoreTokens())
         productID = tokenizer.nextToken();
      if(null == merchant || null == productID)
         return merchant;
      else
         return merchant.getProduct(productID);
   }
}