package com.psol.xcommerce;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Comlet is a subset of the servlet interface, Shop 
 * delegates HTTP requests to Comlet descendants.
 *
 * @version Sep 10, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class Comlet
   implements ServletConfig
{
   /**
    * properties
    */
   protected ServletConfig servletConfig;
   protected long lastModified = -1;
   protected Shop shop;

   /**
    * creates a new Comlet
    * @param shop the shop it is part of
    */
   public Comlet(Shop shop)
   {
      this.shop = shop;
   }

   /**
    * return the servlet config
    */
   public ServletConfig getServletConfig()
   {
      return shop.getServletConfig();
   }

   /**
    * return the shop it is part of
    */
   public Shop getShop()
   {
      return shop;
   }

   /**
    * convenience: implements ServletConfig
    * @param name the parameter whose value is requested
    * @return the parameter value
    */
   public String getInitParameter(String name)
   {
      return shop.getInitParameter(name);
   }

   /**
    * convenience: implements ServletConfig
    * @return an enumeration of names
    */   
   public Enumeration getInitParameterNames() 
   {
      return shop.getInitParameterNames();
   }

   /**
    * convenience: implements ServletConfig
    * @return the config object
    */
   public ServletContext getServletContext()
   {
      return shop.getServletContext();
   }

   /**
    * handle GET request
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   public void doGet(HttpServletRequest request,
                     HttpServletResponse response)
      throws IOException, ServletException
   {
      response.sendError(HttpServletResponse.SC_BAD_REQUEST);
   }

   /**
    * handle POST request
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   public void doPost(HttpServletRequest request,
                      HttpServletResponse response)
      throws IOException, ServletException
   {
      response.sendError(HttpServletResponse.SC_BAD_REQUEST);
   }

   /**
    * return the time the data was last modified
    */
   public long getLastModified()
   {
      return lastModified;
   }

   /**
    * data has changed update lastModified
    */
   public void freshened()
   {
      lastModified = System.currentTimeMillis();
   }
}