package com.psol.xcommerce;

import java.io.*;
import java.text.*;
import org.w3c.dom.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * a product is various information like price
 *
 * @version Sep 10, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class Product
   extends Comlet
{
   /**
    * properties
    */
   protected Element productElement;
   protected Document productDocument;
   protected Merchant merchant;

   /**
    * creates a new product
    * @param element XML description of the product
    * @param merchant merchant owning this product
    * @param shop the shop it is part of
    */
   public Product(Element element,
                  Merchant merchant,
                  Shop shop)
   {
      super(shop);
      productElement = element;
      this.merchant = merchant;
   }

   /**
    * returns the DOM Element
    */
   public Element getElement()
   {
      return productElement;
   }

   /**
    * returns the merchant
    */
   public Merchant getMerchant()
   {
      return merchant;
   }

   /**
    * return the product id
    */
   public String getID()
   {
      return productElement.getAttribute("id");
   }

   /**
    * return the product name
    */
   public String getName()
   {
      Element nameElement =
         XMLUtil.extractFirst(productElement,"name");
      return XMLUtil.getText(nameElement);
   }

   /**
    * return the product price
    */
   public String getPrice()
   {
      Element priceElement =
         XMLUtil.extractFirst(productElement,"price");
      return XMLUtil.getText(priceElement);
   }

   /**
    * return an document around the product
    */
   protected Document getDocument()
   {
      if(null == productDocument)
      {
         productDocument = XMLUtil.createDocument(productElement);
         Element element = productDocument.getDocumentElement();
         element.setAttribute("merchant",merchant.getID());
      }
      return productDocument;
   }

   /**
    * handle GET request
    * @param request the request received from the client
    * @param response interface to the client
    * @exception IOException error writing the reply
    * @exception ServletException error in processing the request
    */
   public void doGet(HttpServletRequest request,
                     HttpServletResponse response)
      throws IOException, ServletException
   {
      XMLUtil.transform(getDocument(),
                        merchant.getProductXSL(),
                        response.getWriter(),
                        response.getCharacterEncoding());
   }

   /**
    * return the time the data was last modified
    */
   public long getLastModified()
   {
      return merchant.getLastModified();
   }   
}