package com.psol.xcommerce;

import java.io.*;
import java.net.*;
import java.util.*;

/**
 * Does an HTTP POST.
 *
 * @version Sep 10, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class HTTPPost
{
   /**
    * properties
    */
   protected URL url;
   protected String query;

   /**
    * Creates a new HTTPPost.
    * @param url URL to connect to
    * @parameters POST request parameter
    */
   public HTTPPost(URL url,Dictionary parameters)
   {
      this.url = url;
      query = buildQuery(parameters);
   }

   /**
    * Creates a new HTTPPost.
    * @param url URL to connect to
    * @parameters POST request parameter
    * @exception MalformedURLException if the URL is invalid
    */
   public HTTPPost(String url,Dictionary parameters)
      throws MalformedURLException
   {
      this(new URL(url),parameters);
   }

   /**
    * executes the post request
    * @exception IOException error posting the data
    */
   public void doRequest()
      throws IOException
   {
      // this stupid thing does not default to 80...
      int port = url.getPort();
      if(port == -1)
         port = 80;
      Socket s = new Socket(url.getHost(),port);
      PrintStream o = new PrintStream(s.getOutputStream());
      o.print("POST "); o.print(url.getFile());
      o.print(" HTTP/1.0\r\n");
      o.print("Accept: text/html text/xml\r\n");
      o.print("Host: "); o.print(url.getHost()); o.print("\r\n");
      o.print("Content-type: ");
      o.print("application/x-www-form-urlencoded\r\n");
      o.print("Content-length: "); o.print(query.length());
      o.print("\r\n\r\n");
      o.print(query);
      o.print("\r\n");
      InputStream i = s.getInputStream();
      StringBuffer reply = new StringBuffer();
      int c = i.read();
      boolean firstLine = true;
      while(c != -1)
      {
         if(firstLine)
            if(c == '\r' || c == '\n')
               firstLine = false;
            else
               reply.append((char)c);
         c = i.read();
      }
      String stReply = reply.toString();
      int returnCode = Integer.parseInt(stReply.substring(9,12));
      if(!(returnCode >= 200 && returnCode < 300))
         throw new ProtocolException(stReply.substring(13));
   }

   /**
    * format the request according to the proper encoding
    * @param parameters query parameters
    * @return string with the query properly formatted
    */
   protected String buildQuery(Dictionary parameters)
   {
      StringBuffer request = new StringBuffer();
      Enumeration keys = parameters.keys();
      String key = null;
      boolean first = true;
      while(keys.hasMoreElements())
      {
         if(!first)
            request.append('&');
         else
            first = false;
         key = (String)keys.nextElement();
         request.append(key);
         request.append('=');
         request.append(
            URLEncoder.encode((String)parameters.get(key)));
         // request.append("\r\n");
      }
      return request.toString();
   }
}