package com.psol.xcommerce;

import java.io.*;
import java.util.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import com.lotus.xsl.*;
import javax.servlet.*;
import com.ibm.xml.dom.*;
import com.ibm.xml.parsers.*;
import com.lotus.xml.*;
import com.lotus.xml.xml4j2dom.*;

/**
 * XMLUtil isolates non-portable aspects of DOM, calling
 * XSL processor and some utility functions.<BR>
 * This version is for IBM's XML for Java, to use another
 * processor this is the only class to change.
 *
 * @version Sep 10, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

class XMLUtil
{
   /**
    * the HTML doctype
    */
   protected static final String DOCTYPE =
      "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 " +
      "Transitional//EN\">";

   /**
    * parses a document with DOM
    * @param systemID system id (URL) for the document
    * @return DOM Document
    * @exception ServletException error parsing the document
    */
   public static Document parse(String systemID)
      throws ServletException
   {
      try
      {
         DOMParser parser = new DOMParser();
         parser.parse(systemID);
         return parser.getDocument();
      }
      catch(SAXException e)
      {
         throw new ServletException(e);
      }
      catch(IOException e)
      {
         throw new ServletException(e);
      }
   }

   /**
    * parses a document with DOM
    * @param reader reader for the document
    * @return DOM Document
    * @exception ServletException error parsing the document
    */
   public static Document parse(Reader reader)
      throws ServletException
   {
      try
      {
         InputSource inputSource = new InputSource(reader);
         DOMParser parser = new DOMParser();
         parser.parse(inputSource);
         return parser.getDocument();
      }
      catch(SAXException e)
      {
         throw new ServletException(e);
      }
      catch(IOException e)
      {
         throw new ServletException(e);
      }
   }

   /**
    * turns DOM tree in a string
    * @param node root of the tree
    */
   public static String toString(Node node)
   {
      // IBM parser has no "saveAs" method or we would use it
      // this version does not handle all node types
      StringBuffer buffer = new StringBuffer();
      switch(node.getNodeType())
      {
         case Node.ATTRIBUTE_NODE:
         {
            Attr attr = (Attr)node;
            buffer.append(' ');
            buffer.append(attr.getName());
            buffer.append("=\"");
            buffer.append(attr.getValue());
            buffer.append('\"');
            break;
         }
         case Node.DOCUMENT_NODE:
         {
            Document document = (Document)node;
            Element topLevel = document.getDocumentElement();
            buffer.append("<?xml version=\"1.0\"?>");
            buffer.append(toString(topLevel));
            break;
         }
         case Node.ELEMENT_NODE:
         {
            Element element = (Element)node;
            buffer.append("<");
            buffer.append(element.getTagName());
            NamedNodeMap attrs = element.getAttributes();
            for(int i = 0;i < attrs.getLength();i++)
               buffer.append(toString(attrs.item(i)));
            NodeList children = node.getChildNodes();
            if(children.getLength() == 0)
               // shorthand for empty element
               buffer.append("/>");
            else
            {
               buffer.append(">");
               for(int i = 0;i < children.getLength();i++)
                  buffer.append(toString(children.item(i)));
               buffer.append("</");
               buffer.append(element.getTagName());
               buffer.append(">");
            }
            break;
         }
         case Node.TEXT_NODE:
         {
            Text text = (Text)node;
            buffer.append(text.getData());
            break;
         }
         default:
            throw new NotImplementedError();
      }
      return buffer.toString();
   }

   /**
    * creates an empty DOM document
    * @return DOM document
    */
   public static Document createDocument()
   {
      Document document = new DocumentImpl();
      return document;
   }

   /**
    * creates a DOM document with a top-level element
    * @param element top-level element
    * @return DOM document
    */
   public static Document createDocument(Element element)
   {
      Document document = createDocument();
      Node celement = cloneNode(document,element);
      document.appendChild(celement);
      return document;
   }

   /**
    * clone a DOM Node in the context of a Document<BR>
    * Element.cloneNode() does not work when copying elements
    * from one document to another, the clone remains attached
    * to original document
    * @param document document to attach the clone to
    * @param node node to clone
    * @return the clone
    */
   public static Node cloneNode(Document document,
                                Node node)
   {
      // o = original
      // c = clone
      switch(node.getNodeType())
      {
         case Node.ATTRIBUTE_NODE:
         {
            Attr o = (Attr)node,
                 c = document.createAttribute(o.getName());
            c.setValue(o.getValue());
            return c;
         }
         case Node.ELEMENT_NODE:
         {
            Element o = (Element)node,
                    c = document.createElement(o.getTagName());
            NodeList children = o.getChildNodes();
            for(int i = 0;i < children.getLength();i++)
            {
               Node n = cloneNode(document,children.item(i));
               c.appendChild(n);
            }
            NamedNodeMap attrs = o.getAttributes();
            for(int i = 0;i < attrs.getLength();i++)
            {
               Attr a = (Attr)cloneNode(document,attrs.item(i));
               c.setAttributeNode(a);
            }
            return c;
         }
         case Node.TEXT_NODE:
         {
            Text o = (Text)node,
                 c = document.createTextNode(o.getData());
            return c;
         }
         default:
            throw new NotImplementedError();
      }
   }

   /**
    * apply a style sheet and prints the result
    * @param document original document
    * @param xsl style sheet
    * @param writer output writer
    * @param encoding character encoding
    */
   public static void transform(Document document,
                                Document xsl,
                                PrintWriter writer,
                                String encoding)
      throws ServletException
   {
      XML4JLiaison4dom liaison = new XML4JLiaison4dom();
      XSLTInputSource documentIn = new XSLTInputSource(document),
                      xslIn = new XSLTInputSource(xsl);
      XSLTResultTarget result = new XSLTResultTarget(writer);
      try
      {
         XSLProcessor xslProcessor = new XSLProcessor(liaison);
         xslProcessor.process(documentIn,xslIn,result);
      }
      catch(Exception e)
      {
          throw new ServletException(e);
      }
   }

   /**
    * retrieve elements in the children of a node<BR>
    * it assumes no recursive structure, in other
    * word in products/product/related/product,
    * it would NOT find the second product
    * @param element top of tree
    * @param name element we are looking for
    * @return an enumeration of the elements found
    */
   public static Enumeration extract(Element element,String name)
   {
      // ultimately replace with a XQL engine
      Vector vector = new Vector();
      extract(element,name,vector);
      return vector.elements();
   }

   /**
    * retrieve the first element in the children of a node
    * @param element top of tree
    * @param name element we are looking for
    * @return an enumeration of the elements found
    */
   public static Element extractFirst(Element element,
                                      String name)
   {
      Vector vector = new Vector();
      extract(element,name,vector);   // not optimized
      if(vector.size() > 0)
         return (Element)vector.firstElement();
      else
         return null;
   }

   /**
    * helper method for extract
    * @param node top of tree
    * @param name element we are looking for
    * @return an enumeration of the elements found
    */
   protected static void extract(Node node,
                                 String name,
                                 Vector vector)
   {
      if(node.getNodeType() == Node.ELEMENT_NODE)
      {
         if(node.getNodeName().equals(name))
            // we stop, so it does not work with
            // recursive structures
            vector.addElement(node);
         else
         {
            NodeList children = node.getChildNodes();
            for(int i = 0;i < children.getLength();i++)
               extract(children.item(i),name,vector);
         }
      }
   }

   /**
    * returns the text in the node
    * @param node the node to read from
    * @return the text in the node
    */
   public static String getText(Node node)
   {
      StringBuffer text = new StringBuffer();
      if(node.getNodeType() == Node.ELEMENT_NODE)
      {
         NodeList children = node.getChildNodes();
         for(int i = 0;i < children.getLength();i++)
         {
            Node n = children.item(i);
            if(n.getNodeType() == Node.TEXT_NODE)
            {
               Text t = (Text)n;
	           text.append(t.getData());
            }
         }
      }
      return text.toString();
   }

   /**
    * check if a string is empty
    * @param string string to test
    * @return true if empty, false otherwise
    */
   public static boolean isEmpty(String string)
   {
      if(null != string)
         return string.trim().length() == 0;
      else
         return true;
   }

   /**
    * Write the input in XML (or HTML) by escaping what
    * must be escaped.
    * @param writer write on this
    * @param string string to write
    * @exception IOException error writing
    */
   public static void writeInXML(Writer writer,String string)
      throws IOException
   {
      for(int i = 0;i < string.length();i++)
      {
         char c = string.charAt(i);
         if(c == '<')
            writer.write("&lt;");
         else if(c == '&')
            writer.write("&amp;");
         else
            writer.write(c);
      }
   }
}