package com.psol.xcommerce;

import java.io.*;
import org.w3c.dom.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * Editor is a web-tool to create product lists
 * for smaller merchants.
 *
 * @version Sep 10, 1999
 * @author Benot Marchal <bmarchal@pineapplesoft.com>
 */

public class Editor
   extends HttpServlet
{
   /**
    * the editor's style sheet
    */
   protected Document styleSheet;

   /**
    * initializes the servlet, read the style sheet
    * @exception could not read the style sheet
    */
   public void init()
      throws ServletException
   {
      String fname = getInitParameter("editor.xsl");
      styleSheet = XMLUtil.parse(fname);
   }

   /**
    * process GET requests
    * @param request request received from the client
    * @param response response to the client
    */
   protected void doGet(HttpServletRequest request,
                        HttpServletResponse response)
      throws IOException, ServletException
   {
      String merchant = request.getParameter("merchant"),
             fname = getInitParameter(merchant + ".xml");
      if(null == merchant || null == fname)
      {
         response.sendError(HttpServletResponse.SC_NOT_FOUND);
         return;
      }
      Document document = XMLUtil.parse(fname);
      XMLUtil.transform(document,
                        styleSheet,
                        response.getWriter(),
                        response.getCharacterEncoding());
   }

   /**
    * handle POST method, HttpServlet forward POST request from service()
    * to this method
    * @param request the request received from the client
    * @param response interface to the client
    */
   protected void doPost(HttpServletRequest request,
                         HttpServletResponse response)
      throws IOException, ServletException
   {
      String merchant = request.getParameter("merchant"),
             fname = getInitParameter(merchant + ".xml");
      if(null == merchant || null == fname)
      {
         response.sendError(HttpServletResponse.SC_NOT_FOUND);
         return;
      }
      String pwdRequest = request.getParameter("pwd"),
             pwdCheck = getInitParameter(merchant + ".pwd"),
             xml = request.getParameter("xmldata");
      if(null != pwdCheck && !pwdCheck.equals(pwdRequest))
      {
         response.sendError(HttpServletResponse.SC_FORBIDDEN);
         return;
      }
      Writer writer = new FileWriter(fname);
      writer.write(xml);
      writer.close();
      writer = response.getWriter();
      writer.write("<HTML><HEAD><TITLE>Confirmation");
      writer.write("</TITLE></HEAD><BODY><P>");
      writer.write("Your changes were saved as follow:<PRE>");
      XMLUtil.writeInXML(writer,xml);
      writer.write("<PRE></BODY></HTML>");
      writer.flush();
   }
}